<?php
namespace Network;

use App\Router\Session;
use Model\User;
use Model\Athlete;
use Model\CoachAthlete;
use Repository\IUserRepository;
use Shared\IHashPassword;
use Shared\Log;
const USER = 'USER';
class AuthService implements IAuthService
{
    private IUserRepository $userRepository;
    private IHashPassword $passwordHacher;

    private ?User $currentUser = null;

    public function __construct(IUserRepository $userRepository, IHashPassword $passwordHacher)
    {
        $this->userRepository = $userRepository;
        $this->passwordHacher = $passwordHacher;
    }

    public function login(string $email, string $password): bool
    {
        $user = $this->userRepository->getItemByEmail($email);
        if ($user === null || !$this->passwordHacher->isPasswordValid( $user->getMotDePasse(),$password)) {
            return false;
        }
        $this->currentUser = $user;
        Session::getInstance()->__set(USER, $this->currentUser->getId());
        return true;
    }



    public function register(string $email, string $password, $data): bool
    {
        if ($this->userRepository->getItemByEmail($email)) {
            throw new \Exception('User already exists');
        }

        $hashedPassword = $this->passwordHacher->hashPassword($password);
        $prenom = $data['prenom'];
        $username = $data['username'];
        $nom = $data['nom'];
        $sexe = $data['sexe'];
        $taille = $data['taille'];
        $poids = $data['poids'];
        $dateNaissance = $data['dateNaissance'];
        $roleName = $data['roleName'];
        $role = null;
        if ($roleName == "Coach") {
            $role = new CoachAthlete();
        } else if ($roleName == "Athlete") {
            $role = new Athlete();
        }

        $user = new User(
            random_int(0, 100),
            $nom,
            $prenom,
            $username,
            $email,
            $hashedPassword,
            $sexe,
            $taille,
            $poids,
            $dateNaissance,
            //should use reflexion
            $role
        );
        $this->userRepository->addItem($user);
        $this->currentUser = $user;
        Session::getInstance()->__set(USER,$this->currentUser->getId());


        return true;
    }

    public function logoutUser(): bool
    {
        $this->currentUser = null;
        Session::getInstance()->destroy;
        return true;
    }
    
    public function getCurrentUser(): ?User
    {
        if (!empty(Session::getInstance()->__get(USER)) && $this->currentUser === null) {
            $this->currentUser = $this->userRepository->getItemById(Session::getInstance()->__get(USER));
        }
        return $this->currentUser;
    }


}
