<?php

namespace Stub;

use Model\Athlete;
use Model\CoachAthlete;
use Model\User;
use Repository\IUserRepository;

class UserRepository implements IUserRepository {
    private array $users = [];

    public function __construct() {
        $this->users[] = new User(1, "Doe", "John", "Doe","john.doe@example.com", '$2y$10$U59ioMTGZBM2FGQv.3lcbuL0IkO4Fx1jQU7f5hF7o/hvCX2t46mby', 'M', 1.80, 75, new \DateTime("1985-05-15"), new CoachAthlete());
        $this->users[] = new User(2, "Smith", "Jane","Smith", "jane.smith@example.com", '$2y$10$U59ioMTGZBM2FGQv.3lcbuL0IkO4Fx1jQU7f5hF7o/hvCX2t46mby', 'F', 1.65, 60, new \DateTime("1990-03-10"), new Athlete());
        $this->users[] = new User(3, "Martin", "Paul","Martin", "paul.martin@example.com", '$2y$10$U59ioMTGZBM2FGQv.3lcbuL0IkO4Fx1jQU7f5hF7o/hvCX2t46mby', 'M', 1.75, 68, new \DateTime("1988-08-20"), new CoachAthlete());
        $this->users[] = new User(4, "Brown", "Anna","Brown", "anna.brown@example.com", '$2y$10$U59ioMTGZBM2FGQv.3lcbuL0IkO4Fx1jQU7f5hF7o/hvCX2t46mby', 'F', 1.70, 58, new \DateTime("1992-11-25"), new Athlete());
        $this->users[] = new User(5, "Lee", "Bruce","Lee", "bruce.lee@example.com", '$2y$10$U59ioMTGZBM2FGQv.3lcbuL0IkO4Fx1jQU7f5hF7o/hvCX2t46mby', 'M', 1.72, 70, new \DateTime("1970-02-05"), new Athlete());
    }

    public function getItemById(int $id): ?User {
        foreach ($this->users as $user) {
            if ($user->getId() === $id) {
                return $user;
            }
        }
        return null;
    }

    public function getItemByEmail(string $email): ?User {
        foreach ($this->users as $user) {
            if ($user->getEmail() === $email) {
                return $user;
            }
        }
        return null;
    }

    public function getNbItems(): int {
        return count($this->users);
    }

    public function getItems(int $index, int $count, ?string $orderingPropertyName = null, bool $descending = false): array {
        // Cette méthode est un exemple simple, on ne gère pas l'ordonnancement ici
        return array_slice($this->users, $index, $count);
    }

    public function getItemsByName(string $substring, int $index, int $count, ?string $orderingPropertyName = null, bool $descending = false): array {
        $filteredUsers = array_filter($this->users, function ($user) use ($substring) {
            return str_contains(strtolower($user->getNom()), strtolower($substring)) || str_contains(strtolower($user->getPrenom()), strtolower($substring));
        });
        return array_slice($filteredUsers, $index, $count);
    }
    public function getItemByName(string $substring, int $index, int $count, ?string $orderingPropertyName = null, bool $descending = false): ?User {
        $filteredUsers = $this->GetItemsByName($substring, $index, $count, $orderingPropertyName, $descending);
        return $filteredUsers[0] ?? null;
    }

    // should have type here
    public function updateItem($oldUser, $newUser): void {
        $index = array_search($oldUser, $this->users);
        if ($index !== false) {
            $this->users[$index] = $newUser;
        }
    }

    // should have type here
    public function addItem( $user): void {
        $this->users[] = $user;
    }

    // should have type here
    public function deleteItem( $user): bool {
        $index = array_search($user, $this->users);
        if ($index !== false) {
            unset($this->users[$index]);
            return true;
        }
        return false;
    }

    public function addFriend(int $user1, int $user2)
    {
        return true;
    }


    public function deleteFriend(int $user1, int $user2)
    {
       return true;
    }
}

?>